// SPDX-FileCopyrightText: 2015 - 2022 UnionTech Software Technology Co., Ltd.
//
// SPDX-License-Identifier: LGPL-3.0-or-later

#ifndef BUTTON_CONSTANTS_H
#define BUTTON_CONSTANTS_H

#include <dtkwidget_global.h>

DWIDGET_BEGIN_NAMESPACE

namespace DIALOG {
    const int DEFAULT_WIDTH = 380;
    const int DEFAULT_HEIGHT = 120;
    const int BORDER_SHADOW_WIDTH = 0;
    const int BORDER_RADIUS = 4;
    const int CONTENT_INSERT_OFFSET = 2;
    const int BUTTON_HEIGHT = 28;
    const int CLOSE_BUTTON_WIDTH = 21;
    const int CLOSE_BUTTON_HEIGHT = 21;
    const int ICON_LAYOUT_TOP_MARGIN = 14;
    const int ICON_LAYOUT_BOTTOM_MARGIN = 14;
    const int ICON_LAYOUT_LEFT_MARGIN = 20;
    const int ICON_LAYOUT_RIGHT_MARGIN = 20;
    const int ICON_LAYOUT_SPACING = 20;
    const int BUTTON_LAYOUT_TOP_MARGIN = 0;
    const int BUTTON_LAYOUT_BOTTOM_MARGIN = 10;
    const int BUTTON_LAYOUT_LEFT_MARGIN = 10;
    const int BUTTON_LAYOUT_RIGHT_MARGIN = 10;
}

DWIDGET_END_NAMESPACE

#endif // BUTTON_CONSTANTS_H

